create or replace package TFC_RAP is
  -- Funci que incrementa l' identificador de la taula TFC_RASTRE_LINIA  
  -- Retorn de la funci: L' identificador de la taula TFC_RASTRE_LINIA incrementat 
  FUNCTION INCREMENTAR_IDERAL RETURN NUMBER;

  -- Funci que verifica si s' ha iniciat un rastre de procs  
  -- Retorn de la funci:
  --     TRUE  => Hi ha un rastre iniciat 
  --     FALSE => No hi ha cap rastre iniciat 
  FUNCTION RASTRE_INICIAT RETURN BOOLEAN;
  
  -- Procediment que inicia un rastre de procs  
  -- Parmetres d' entrada:
  --   P_PROCES => Nom del procs per el que s' inicia un rastre 
  PROCEDURE INICIAR_RASTRE(P_PROCES VARCHAR2);
  
  -- Procediment que finalitza correctament un rastre iniciat. Actualitza el valor de TFC_RASTRE_PROCES.ESTAT=FIN   
  PROCEDURE FINALITZAR_RASTRE_OK;

  -- Procediment que finalitza correctament un rastre iniciat. Actualitza el valor de TFC_RASTRE_PROCES.ESTAT=ERR
  PROCEDURE FINALITZAR_RASTRE_ERR;
  
  -- Procediment que registre un detall de rastre a la taula TFC_RASTRE_LINIA 
  -- Parmetres d' entrada:
  --   P_DETALL => Detall del rastre 
  PROCEDURE REGISTRAR_RASTRE_LINIA(P_DETALL VARCHAR2);
  
  -- Funci que retorna l' identificador de rastre actual 
  FUNCTION OBT_IDERAP RETURN TFC_RASTRE_PROCES.IDERAP%TYPE;
end TFC_RAP;
/
create or replace package body TFC_RAP is
  -- Valor del camp TFC_RASTRE_PROCES.ESTAT per un rastre no finalitzat 
  cEstat_Ini constant varchar2(3) := 'INI';
  -- Valor del camp TFC_RASTRE_PROCES.ESTAT per un rastre finalitzat correctament  
  cEstat_Fin constant varchar2(3) := 'FIN';
  -- Valor del camp TFC_RASTRE_PROCES.ESTAT per un rastre finalitzat amb error
  cEstat_Err constant varchar2(3) := 'ERR';

  -- Global que guarda l' identificador d' un rastre iniciat 
  vIdeRap TFC_RASTRE_PROCES.IDERAP%Type;
  -- Global que guarda l' identificador de linia de rastre 
  vIdeRal TFC_RASTRE_LINIA.IDERAL%Type;
  
  PROCEDURE INICIALITZAR_IDERAL IS 
  BEGIN
    vIdeRal := 0;
  END;
  
  -- Funci que incrementa l' identificador de la taula TFC_RASTRE_LINIA  
  -- Retorn de la funci: L' identificador de la taula TFC_RASTRE_LINIA incrementat 
  FUNCTION INCREMENTAR_IDERAL RETURN NUMBER IS
  BEGIN
    vIdeRal := vIdeRal + 1;
    
    return vIdeRal;
  END;
  
  -- Funci que verifica si s' ha iniciat un rastre de procs  
  -- Retorn de la funci:
  --     TRUE  => Hi ha un rastre iniciat 
  --     FALSE => No hi ha cap rastre iniciat 
  FUNCTION RASTRE_INICIAT RETURN BOOLEAN IS 
  BEGIN
    RETURN (vIdeRap is not null);
  END;
  
  -- Procediment que inicia un rastre de procs  
  -- Parmetres d' entrada:
  --   P_PROCES => Nom del procs per el que s' inicia un rastre 
  PROCEDURE INICIAR_RASTRE(P_PROCES VARCHAR2) IS
    PRAGMA AUTONOMOUS_TRANSACTION;
  BEGIN
    -- Si s' ha informat un nom de procs 
    If (P_PROCES is not null) then 
      If (vIdeRap is not null) then -- Si hi ha un rastre iniciat es finalitza 
        Update TFC_RASTRE_PROCES 
           set ESTAT  = cEstat_Fin,
               DATFIN = SYSDATE
           where IDERAP = vIdeRap and 
                 ESTAT  = cEstat_Ini;
      end if;

      -- Es crea un nou rastre de procs     
      Insert into TFC_RASTRE_PROCES
        (IDERAP,
         PROCES,
         DATINI,
         ESTAT)
      values
        (RAP_SEQ.NextVal,
         P_PROCES,
         SYSDATE,
         cEstat_Ini)
      returning IDERAP into vIdeRap; -- S' informa la global per saber quin rastre est obert 
      
      INICIALITZAR_IDERAL;
    end if;
    
    COMMIT;
  END;
  
  -- Procediment que finalitza correctament un rastre iniciat. Actualitza el valor de TFC_RASTRE_PROCES.ESTAT=FIN   
  PROCEDURE FINALITZAR_RASTRE_OK IS
    PRAGMA AUTONOMOUS_TRANSACTION;
  BEGIN
    -- Si hi ha un rastre iniciat es finalitza correctament
    If (vIdeRap is not null) then 
      Update TFC_RASTRE_PROCES 
         set ESTAT  = cEstat_Fin,
             DATFIN = SYSDATE
         where IDERAP = vIdeRap and 
               ESTAT  = cEstat_Ini;
               
      vIdeRap := null;
    end if;
    
    COMMIT;
  END;
  
  -- Procediment que finalitza correctament un rastre iniciat. Actualitza el valor de TFC_RASTRE_PROCES.ESTAT=ERR
  PROCEDURE FINALITZAR_RASTRE_ERR IS
    PRAGMA AUTONOMOUS_TRANSACTION;
  BEGIN
    -- Si hi ha un rastre iniciat es finalitza amb error 
    If (vIdeRap is not null) then 
      Update TFC_RASTRE_PROCES 
         set ESTAT  = cEstat_Err,
             DATFIN = SYSDATE
         where IDERAP = vIdeRap and 
               ESTAT  = cEstat_Ini;
               
      vIdeRap := null;
    end if;
    
    COMMIT;
  END;
  
  -- Procediment que registre un detall de rastre a la taula TFC_RASTRE_LINIA 
  -- Parmetres d' entrada:
  --   P_DETALL => Detall del rastre 
  PROCEDURE REGISTRAR_RASTRE_LINIA(P_DETALL VARCHAR2) IS
    PRAGMA AUTONOMOUS_TRANSACTION;
    vId TFC_RASTRE_LINIA.IDERAL%Type;
  BEGIN
    -- Si hi ha un rastre iniciat i s' informa un detall es crea un registre de linia de rastre 
    If ((P_DETALL is not null) and (vIdeRap is not null)) then 
      vId := INCREMENTAR_IDERAL;
      Insert into TFC_RASTRE_LINIA
        (RAP_IDERAP,
         IDERAL,
         TEXTE)
      values 
        (vIdeRap,
         vId,
         P_DETALL);
  
      COMMIT;
    end if;
  END;
  
  -- Funci que retorna l' identificador de rastre actual 
  FUNCTION OBT_IDERAP RETURN TFC_RASTRE_PROCES.IDERAP%TYPE IS
  BEGIN
    return vIdeRap;
  END;
end TFC_RAP;
/
